/*****************************************************************************
|*
|*  File:    pcm51cc01.h
|*
|*  Version:
|* 
|*  Copyright 2001-2003 Altium BV
|*
 ****************************************************************************/

#ifndef _PCM51CC01_H
#define _PCM51CC01_H
/* T89C51CC01 characteristics */
#define HAS_IEN1                     /* T89C51CC01 has IEN0 and IEN1 */
#define HAS_AUXRAM
#define SIZE_OF_AUXRAM 0x400
/* board characteristics */
#define PHYTEC_MMU_INIT
#define PHYTEC_MMU_V2        /* define before including "phytec.h" */
#define COPY_SERIAL_VECTOR
#include "phytec.h"
#include "data.h"

_inline void
ta_init(void)
{
    CLR_P1_0;                            /* Pull P1.0 to Zero to Turn on LED D3 */
    IEN1 = 0;
    saveIEN1 = 0;
    phytec_mmu_init();
    IPL0 = 0;
    IPH0 = 0;
    IPL1 = 0;
    IPH1 = 0;
    P3   = 0xFF;
}


_inline void
ta_serial_init(void)
{
    /* baudrate selection (using timer 1)
     * baud-rate = 2^SMOD/32 * Fosc/(12 * [256 - ReloadValue]) = 9600 baud
     */
    PCON = 0x80;     /* Serial_smod * 128 */
    SCON = 0x52;     /* Mode 1 */
    TH1 = 0xF5;        /*  9600 Baud +/- 2%                   SERIAL_REL */
    TMOD = 0x20;     /* Timer 1 in mode 2 (8-bit auto-reload) */
    TR1 = 1;              /* Timer 1 on */
    /* set priority */
    IPL0 = 0x10;        /* serial interrupt has highest priority (1) */
    IPL1 = 0x00;
    IPH0 = 0x10;
    IPH1 = 0x00;
    IEN0 = 0x90;        /* Enable Serial interrupt and EA */
}

/*****************************************************************************
|*
|*  FUNCTION:        ta_restore_SFRs
|*
|*  DESCRIPTION:
|*
|*      Restore SFRs to the user program value, when restoring the interrupt
|*    enable registers ensure that the serial interrupt is enabled, however
|*    do not set the global enable flag yet.
|*
 */
_inline void
ta_restore_SFRs( void )
{
    IEN0 = (saveIE | 0x10) & 0x7F; /* set serial interrupt, but disabled EA */
    IEN1 = saveIEN1;
}


/*****************************************************************************
|*
|*  FUNCTION:        ta_save_SFRs
|*
|*  DESCRIPTION:
|*
|*      Save SFRs of the user program value, enable only the serial interrupt
|*
 */
_inline void
ta_save_SFRs( void )
{
    saveIE = IEN0;
    IEN0 = 0x90;                /* disable all interrupts except serial */
    saveIEN1 = IEN1;
    IEN1 = 0;                     /* disable all ien1 related interrupts */
}

/*****************************************************************************
|*
|*  FUNCTION:    ta_write_auxcode
|*  DESCRIPTION:
|*
|*      Write Code to external RAM (XRAM) under the on-chip 1024 
|*      bytes of expanded RAM (ERAM) 
|*      EXTRAM= 0, the ERAM is selected
|*      EXTRAM= 1, the XRAM is selected.
|*/

_inline void
ta_write_auxcode(unsigned int  address, unsigned char value)
{
    if (AUXR & 0x02)              /* =GM= EXTRAM INACTIVE */
    {    
        *(_xdat unsigned char *)address = value;
    }
    else                                /* =GM= EXTRAM ACTIVE*/
    {    
        EA=0;                         /* =GM= DISABLE INTERRUPTS (NO WRITING TO EXTRAM)*/
        SET_EXTRAM;            /* =GM= EXTRAM DISABLE   */
        *(_xdat unsigned char *)address = value;
        CLR_EXTRAM;            /* =GM= EXTRAM ENABLE    */
        EA=1;                         /* =GM= ALLOW INTERRUPTS */
    }
}
#endif /* _PCM51CC01_H */
